@@ -6,10 +6,12 @@ from starlette.responses import JSONResponse
 from starlette.types import ASGIApp, Receive, Scope, Send
 
 
-def _get_json(url):
-    with urllib.request.urlopen(url) as f:
-        data = json.loads(f.read().decode())
-    return data
+def _get_keys(url_or_keys):
+    if isinstance(url_or_keys, str) and url_or_keys.startswith('https://'):
+        with urllib.request.urlopen(url_or_keys) as f:
+            return json.loads(f.read().decode())
+    else:
+        return url_or_keys
 
 
 class InvalidToken(Exception):
@@ -19,10 +21,14 @@ class InvalidToken(Exception):
 
 
 def _validate_provider(provider_name, provider):
-    mandatory_keys = {'issuer', 'uri', 'audience'}
+    mandatory_keys = {'issuer', 'keys', 'audience'}
     if not mandatory_keys.issubset(set(provider)):
         raise ValueError(f'Each provider must contain the following keys: {mandatory_keys}. Provider "{provider_name}" is missing {mandatory_keys - set(provider)}.')
 
+    keys = provider['keys']
+    if isinstance(keys, str) and keys.startswith('http://'):
+        raise ValueError(f'When "keys" is a url, it must start with "https://". This is not true in the provider "{provider_name}"')
+
 
 class AuthenticateMiddleware:
     """
@@ -38,24 +44,23 @@ class AuthenticateMiddleware:
 
     If multiple providers are passed, the request is valid if any of the providers authenticates the request.
     """
-    def __init__(self, app: ASGIApp, providers, public_paths=None, get_json=None) -> None:
+    def __init__(self, app: ASGIApp, providers, public_paths=None, get_keys=None) -> None:
         self._app = app
         for provider in providers:
             _validate_provider(provider, providers[provider])
         self._providers = providers
-        self._get_json = get_json or _get_json
+        self._get_keys = get_keys or _get_keys
         self._public_paths = public_paths or set()
 
-        # cached attributes, as they are retrieved from the internet
-        self._jwks = {}
-        self._metadata = {}
+        # cached attribute
+        self._keys = {}
 
-    def provider_claims(self, provider, token):
+    def _provider_claims(self, provider, token):
         """
         Validates the token and returns its respective claims against a specific provider.
         """
         return jose.jwt.decode(
-            token, self.jwk(provider),
+            token, self._provider_keys(provider),
             issuer=self._providers[provider]['issuer'],
             audience=self._providers[provider]['audience'],
         )
@@ -68,7 +73,7 @@ class AuthenticateMiddleware:
         errors = {}
         for provider in self._providers:
             try:
-                return self.provider_claims(provider, token)
+                return self._provider_claims(provider, token)
             except jose.exceptions.ExpiredSignatureError as e:
                 # if the token has expired, it is at least from this provider.
                 errors = str(e)
@@ -100,18 +105,10 @@ class AuthenticateMiddleware:
 
         return await self._app(scope, receive, send)
 
-    def metadata(self, provider: str):
-        """
-        Returns the metadata of the provider. This is retrieved from the internet and cached at the first retrieval.
-        """
-        if self._metadata.get(provider, None) is None:
-            self._metadata[provider] = self._get_json(self._providers[provider]['uri'])
-        return self._metadata[provider]
-
-    def jwk(self, provider: str):
+    def _provider_keys(self, provider: str):
         """
         Returns the jw keys of the provider. This is retrieved from the internet and cached at the first retrieval.
         """
-        if self._jwks.get(provider, None) is None:
-            self._jwks[provider] = self._get_json(self.metadata(provider)['jwks_uri'])
-        return self._jwks[provider]
+        if self._keys.get(provider, None) is None:
+            self._keys[provider] = self._get_keys(self._providers[provider]['keys'])
+        return self._keys[provider]
